@@ -1,4 +1,3 @@
-// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -8,13 +7,26 @@
  *  Copyright (C) 2002-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -34,8 +46,6 @@
 #include "lib/hci.h"
 #include "lib/hci_lib.h"
 
-#include "src/shared/tty.h"
-
 #include "hciattach.h"
 
 struct uart_t {
@@ -57,8 +67,6 @@ struct uart_t {
 #define ENABLE_PM	1
 #define DISABLE_PM	0
 
-int line_disp = 1;
-
 static volatile sig_atomic_t __io_canceled = 0;
 
 static void sig_hup(int sig)
@@ -120,7 +128,7 @@ int uart_speed(int s)
 		return B3500000;
 #endif
 #ifdef B3710000
-	case 3710000
+	case 3710000:
 		return B3710000;
 #endif
 #ifdef B4000000
@@ -134,10 +142,10 @@ int uart_speed(int s)
 
 int set_speed(int fd, struct termios *ti, int speed)
 {
-	if (cfsetospeed(ti, tty_get_speed(speed)) < 0)
+	if (cfsetospeed(ti, uart_speed(speed)) < 0)
 		return -errno;
 
-	if (cfsetispeed(ti, tty_get_speed(speed)) < 0)
+	if (cfsetispeed(ti, uart_speed(speed)) < 0)
 		return -errno;
 
 	if (tcsetattr(fd, TCSANOW, ti) < 0)
@@ -309,12 +317,6 @@ static int ath3k_pm(int fd, struct uart_t *u, struct termios *ti)
 	return ath3k_post(fd, u->pm);
 }
 
-static int qca(int fd, struct uart_t *u, struct termios *ti)
-{
-        fprintf(stderr,"qca\n");
-        return qca_soc_init(fd, u->speed, u->bdaddr);
-}
-
 static int qualcomm(int fd, struct uart_t *u, struct termios *ti)
 {
 	return qualcomm_init(fd, u->speed, ti, u->bdaddr);
@@ -644,7 +646,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 		fprintf(stderr, "Speed %d too high. Remaining at %d baud\n",
 			u->speed, u->init_speed);
 		u->speed = u->init_speed;
-	} else if (!tty_get_speed(u->speed)) {
+	} else if (u->speed != 57600 && uart_speed(u->speed) == B57600) {
 		/* Unknown speed. Why oh why can't we just pass an int to the kernel? */
 		fprintf(stderr, "Speed %d unrecognised. Remaining at %d baud\n",
 			u->speed, u->init_speed);
@@ -1145,11 +1147,6 @@ struct uart_t uart[] = {
 	{ "ath3k",    0x0000, 0x0000, HCI_UART_ATH3K, 115200, 115200,
 			FLOW_CTL, DISABLE_PM, NULL, ath3k_ps, ath3k_pm  },
 
-	/* QCA ROME */
-        { "qca",    0x0000, 0x0000, HCI_UART_H4, 115200, 115200,
-                        FLOW_CTL, DISABLE_PM, NULL, qca, NULL },
-
-
 	/* QUALCOMM BTS */
 	{ "qualcomm",   0x0000, 0x0000, HCI_UART_H4,   115200, 115200,
 			FLOW_CTL, DISABLE_PM, NULL, qualcomm, NULL },
@@ -1202,9 +1199,6 @@ static int init_uart(char *dev, struct uart_t *u, int send_break, int raw)
 	if (u->flags & AMP_DEV)
 		flags |= 1 << HCI_UART_CREATE_AMP;
 
-	if (!strncmp(u->type, "qca", 3))
-		flags |= 1 << HCI_UART_RESET_ON_INIT;
-
 	fd = open(dev, O_RDWR | O_NOCTTY);
 	if (fd < 0) {
 		perror("Can't open serial port");
@@ -1255,8 +1249,6 @@ static int init_uart(char *dev, struct uart_t *u, int send_break, int raw)
 		goto fail;
 	}
 
-if (line_disp) {
-	fprintf(stderr, "Setting TTY to N_HCI line discipline\n");
 	/* Set TTY to N_HCI line discipline */
 	i = N_HCI;
 	if (ioctl(fd, TIOCSETD, &i) < 0) {
@@ -1273,7 +1265,6 @@ if (line_disp) {
 		perror("Can't set device");
 		goto fail;
 	}
-}
 
 	if (u->post && u->post(fd, u, &ti) < 0)
 		goto fail;
@@ -1312,7 +1303,7 @@ int main(int argc, char *argv[])
 	printpid = 0;
 	raw = 0;
 
-	while ((opt=getopt(argc, argv, "bnpt:s:lrf:")) != EOF) {
+	while ((opt=getopt(argc, argv, "bnpt:s:lr")) != EOF) {
 		switch(opt) {
 		case 'b':
 			send_break = 1;
@@ -1345,11 +1336,6 @@ int main(int argc, char *argv[])
 			raw = 1;
 			break;
 
-		case 'f':
-			line_disp = atoi(optarg);
-			fprintf(stderr, "Line_disp val : %d\n", line_disp);
-			break;
-
 		default:
 			usage();
 			exit(1);
@@ -1494,15 +1480,12 @@ int main(int argc, char *argv[])
 			break;
 	}
 
-if (line_disp) {
 	/* Restore TTY line discipline */
-	fprintf(stderr, "Restoring the Line Discipline driver\n");
 	ld = N_TTY;
 	if (ioctl(n, TIOCSETD, &ld) < 0) {
 		perror("Can't restore line discipline");
 		exit(1);
 	}
-}
 
 	return 0;
 }
